#!/usr/bin/env python3# (C) Copyright 2021 ECMWF.
#
# This software is licensed under the terms of the Apache Licence Version 2.0
# which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
# In applying this licence, ECMWF does not waive the privileges and immunities
# granted to it by virtue of its status as an intergovernmental organisation
# nor does it submit to any jurisdiction.
#
import abc
from datetime import datetime
from typing import Optional, Union

import pandas as pd  # type: ignore
from climetlab.sources import url  # type: ignore

from climetlab_maelstrom_power_production import dataset

from . import merger

__version__ = "0.1.0"

PATTERN_1 = "{type}_{date}_00.nc"
PATTERN_2 = "{type}_{date}_12.nc"

DATE_FORMAT = "%Y-%m-%d"
DATE_FORMAT_REMOTE = "%Y%m%d"

AVAILABLE_DATA_DATES = pd.date_range(
    start=datetime(2019, 1, 1), end=datetime(2019, 7, 31)
)


class IncorrectDateFormatException(Exception):
    """Given date has an invalid format."""


class DateUnavailableException(Exception):
    """No data available for given date."""


class Weather(dataset.AbstractDataset):
    """Access the weather data.

    Parameters
    ----------
    date : str or list[str], default None
        Date(s) for which to get the weather data.
        If `None`, all available dates will be fetched.

    """

    _merger: Optional[merger.WeatherMerger] = None

    url_pattern = PATTERN_1
    dates = AVAILABLE_DATA_DATES

    def __init__(self, date: Union[str, list[str]] = None):
        """Initialize and load the dataset."""
        self.date = _convert_dates(date) if date is not None else self.dates
        self.source = self._get_data()

    @abc.abstractproperty
    @property
    def type(self) -> str:
        """Return the type of the weather data.

        Either ml (model level) or pl (pressure level).

        """

    @property
    def merger(self) -> Optional[merger.WeatherMerger]:
        """Get the merger for the weather data."""
        # Using a merger for a single date causes a ValueError from CliMetLab
        if len(self.date) <= 1:
            return None
        if self._merger is None:
            self._merger = merger.WeatherMerger()
        return self._merger

    def _get_data(self) -> url.Url:
        dates = [date.strftime(DATE_FORMAT_REMOTE) for date in self.date]
        return self._load_source(
            type=self.type,
            date=dates,
        )


def _convert_dates(dates: Union[str, list[str]]) -> list[datetime]:
    if isinstance(dates, str):
        dates_as_datetime = [_convert_to_datetime(dates)]  # type: ignore
    else:
        dates_as_datetime = [_convert_to_datetime(date) for date in dates]
    _check_dates_availability(dates_as_datetime)
    return sorted(dates_as_datetime)


def _convert_to_datetime(date: str) -> datetime:
    try:
        return datetime.strptime(date, DATE_FORMAT)
    except ValueError:
        raise IncorrectDateFormatException(
            f"Date {date} has incorrect format ({DATE_FORMAT} required)"
        )


def _check_dates_availability(dates: list[datetime]) -> None:
    for date in dates:
        if not _date_is_available(date):
            raise DateUnavailableException(
                f"Date {date} is not within available dates for the dataset"
            )


def _date_is_available(date: datetime) -> bool:
    return date in AVAILABLE_DATA_DATES
